using UnityEditor;
using System;
using System.Linq;
using System.IO;


static class Builder {

    // private const string BUILD_OPTIONS_ENV_VAR = "BuildOptions";
    private const string SCRIPTING_BACKEND_ENV_VAR = "SCRIPTING_BACKEND";

    static string[] GetEnabledScenes() {
        return (
            from scene in EditorBuildSettings.scenes
            where scene.enabled
            where !string.IsNullOrEmpty(scene.path)
            select scene.path
        ).ToArray();
    }

    // Mystery to me how this works
    static bool TryConvertToEnum<TEnum>(this string strEnumValue, out TEnum value) {
        if (!Enum.IsDefined(typeof(TEnum), strEnumValue)) {
            value = default;
            return false;
        }

        value = (TEnum)Enum.Parse(typeof(TEnum), strEnumValue);
        return true;
    }


    static bool TryGetEnv(string key, out string value) {
        value = Environment.GetEnvironmentVariable(key);
        return !string.IsNullOrEmpty(value);
    }

    // Hopefully this isnt needed
    // static BuildOptions GetBuildOptions() {
    //     if (TryGetEnv(BUILD_OPTIONS_ENV_VAR, out string envVar)) {
    //         string[] allOptionVars = envVar.Split(',');
    //         BuildOptions allOptions = BuildOptions.None;
    //         BuildOptions option;
    //         string optionVar;
    //         int length = allOptionVars.Length;

    //         Console.WriteLine($":: Detecting {BUILD_OPTIONS_ENV_VAR} env var with {length} elements ({envVar})");

    //         for (int i = 0; i < length; i++) {
    //             optionVar = allOptionVars[i];

    //             if (optionVar.TryConvertToEnum(out option)) {
    //                 allOptions |= option;
    //             }
    //             else {
    //                 Console.WriteLine($":: Cannot convert {optionVar} to {nameof(BuildOptions)} enum, skipping it.");
    //             }
    //         }

    //         return allOptions;
    //     }

    //     return BuildOptions.None;
    // }

    static string GetFixedBuildPath(BuildTarget buildTarget, string buildPath, string buildName){
        if (buildTarget.ToString().ToLower().Contains("windows")) {
            buildName += ".exe";
        }
        return buildPath + buildName;
    }

    static void SetScriptingBackendFromEnv(BuildTarget platform) {
        var targetGroup = BuildPipeline.GetBuildTargetGroup(platform);
        if (TryGetEnv(SCRIPTING_BACKEND_ENV_VAR, out string scriptingBackend)) {
            if (scriptingBackend.TryConvertToEnum(out ScriptingImplementation backend)) {
                Console.WriteLine($":: Setting ScriptingBackend to {backend}");
                PlayerSettings.SetScriptingBackend(targetGroup, backend);
            } else {
                string possibleValues = string.Join(", ", Enum.GetValues(typeof(ScriptingImplementation)).Cast<ScriptingImplementation>());
                throw new Exception($"Could not find '{scriptingBackend}' in ScriptingImplementation enum. Possible values are: {possibleValues}");
            }
        } else {
            var defaultBackend = PlayerSettings.GetDefaultScriptingBackend(targetGroup);
            Console.WriteLine($":: Using project's configured ScriptingBackend (should be {defaultBackend} for tagetGroup {targetGroup}");
        }
    }

    static string GetArgument(string name) {
        string[] args = Environment.GetCommandLineArgs();
        for (int i = 0; i < args.Length; i++) {
            if (args[i].Contains(name)) {
                return args[i + 1];
            }
        }
        return null;
    }

    static string GetBuildPath() {
        string buildPath = GetArgument("customBuildPath");
        Console.WriteLine(":: Received customBuildPath " + buildPath);
        if (buildPath == "") {
            throw new Exception("customBuildPath argument is missing");
        }
        return buildPath;
    }

    static string GetBuildName() {
        string buildName = GetArgument("customBuildName");
        Console.WriteLine(":: Received customBuildName " + buildName);
        if (buildName == "") {
            throw new Exception("customBuildName argument is missing");
        }
        return buildName;
    }

    static BuildTarget GetBuildTarget() {
        string buildTargetName = GetArgument("customBuildTarget");
        Console.WriteLine(":: Received customBuildTarget " + buildTargetName);

        if (buildTargetName.TryConvertToEnum(out BuildTarget target)) {
            return target;
        }
        else {
            throw new Exception("customBuildTarget argument invalid");
        }

    }

    static void build() {

        var buildPath      = GetBuildPath();
        var buildName      = GetBuildName();
        var buildTarget    = GetBuildTarget();
        var buildOptions   = BuildOptions.None; // GetBuildOptions();
        var fixedBuildPath = GetFixedBuildPath(buildTarget, buildPath, buildName);

        SetScriptingBackendFromEnv(buildTarget);

        BuildPipeline.BuildPlayer(GetEnabledScenes(), fixedBuildPath, buildTarget, buildOptions);
    }
}
