#!/usr/bin/env sh

UNITY_VER="2020.2.7f1"

# JSON of available versions
# https://public-cdn.cloud.unity3d.com/hub/prod/releases-linux.json


while test $# -gt 0; do
    case "$1" in
        --win) PLATFORM="windows"; shift; ;;
        --lin) PLATFORM="linux"; shift; ;;
        --docker) PLATFORM="docker"; shift; ;;
        --webgl) BUILD_TARGET="WebGL"; shift; ;;
        --win64) BUILD_TARGET="StandaloneWindows64"; shift; ;;
        --lin64) BUILD_TARGET="StandaloneLinux64"; shift; ;;
        # --osx) BUILD_TARGET="StandaloneOSX"; shift ;;
        *) echo "$1 is not a valid value"; shift; continue ;;
    esac
done

export BUILD_TARGET
export BUILD_NAME=${BUILD_NAME:-"Bot"}

if [[ -z $PLATFORM ]]; then echo "Please provide platform this is run from (--win, --lin)"; exit ; fi
if [[ -z $BUILD_TARGET ]]; then echo "Please provide build target (--win64, --lin64, --webgl)"; exit ; fi
# if [[ -z $BUILD_NAME ]]; then echo "Please provide build name (webgl, win, etc.) -n BUILD_NAME"; exit ; fi

if [[ "$PLATFORM" == "windows" ]]; then
    export UNITY_EXECUTABLE="/mnt/c/Program Files/Unity/Hub/Editor/${UNITY_VER}/Editor/Unity.exe"
    export PROJECT_PATH="."
    export BUILD_PATH="./Builds/${BUILD_NAME}_${BUILD_TARGET}/"
fi
if [[ "$PLATFORM" == "linux" ]]; then
    export UNITY_EXECUTABLE="/opt/Unity/Hub/Editor/${UNITY_VER}/Editor/Unity"
    export PROJECT_PATH="$(pwd)"
    export BUILD_PATH="$(pwd)/Builds/${BUILD_NAME}_${BUILD_TARGET}/"
fi
if [[ "$PLATFORM" == "docker" ]]; then
    export UNITY_EXECUTABLE="/usr/bin/unity-editor"
    export PROJECT_PATH="$(pwd)"
    export BUILD_PATH="$(pwd)/Builds/${BUILD_NAME}_${BUILD_TARGET}/"
fi

mkdir -p $BUILD_PATH

# Note sure if this helped or not
export GZIP=-f


bash initbuild.sh
